"""Script for running tsunami inundation scenario for Dampier, WA, Australia.

Source data such as elevation and boundary data is assumed to be available in
directories specified by project.py
The output sww file is stored in project.outputtimedir 

The scenario is defined by a triangular mesh created from project.polygon,
the elevation data and a simulated submarine landslide.

Ole Nielsen and Duncan Gray, GA - 2005 and Jane Sexton, Nick Bartzis, GA - 2006
"""

#------------------------------------------------------------------------------
# Import necessary modules
#------------------------------------------------------------------------------

# Standard modules
from os import sep
from os.path import dirname, basename
from os import mkdir, access, F_OK
from shutil import copy
import time
import sys


# Related major packages
from anuga.shallow_water import Domain
from anuga.shallow_water import Dirichlet_boundary
from anuga.shallow_water import File_boundary
from anuga.shallow_water import Reflective_boundary

from anuga.pmesh.mesh_interface import create_mesh_from_regions

from anuga.geospatial_data.geospatial_data import *

# Application specific imports
import project                 # Definition of file names and polygons



#------------------------------------------------------------------------------
# Copy scripts to time stamped output directory and capture screen
# output to file
#------------------------------------------------------------------------------

# filenames
meshname = project.meshname+'.msh'
source_dir = project.boundarydir
boundary_file = project.boundaryname

from anuga.shallow_water.data_manager import urs2sww



# creates copy of code in output dir if dir doesn't exist
if access(project.outputtimedir,F_OK) == 0:
    print 'project.outputtimedir',dirname(project.outputtimedir)
    mkdir (project.outputtimedir,0777)
copy (dirname(project.__file__) +sep+ project.__name__+'.py',
      project.outputtimedir + project.__name__+'.py')
copy (__file__, project.outputtimedir + basename(__file__))
print 'project.outputtimedir',project.outputtimedir



#--------------------------------------------------------------------------
# Create the triangular mesh based on overall clipping polygon with a
# tagged
# boundary and interior regions defined in project.py along with
# resolutions (maximal area of per triangle) for each polygon
#--------------------------------------------------------------------------


interior_regions = [#[project.karratha_polygon, 25000],
                    #[project.dampier_polygon, 2000],
                    #[project.refinery_polygon, 2000],
                    #[project.point_polygon, 2000]]
                    #[project.cipma_polygon, 20000]]
                    [project.cipma_polygon, 40000]]    # Caused memory error?

#---------------------------
# this is check that no interior polygon is outside the bounding poly
#------------------------------

"""
count = 0
for i in range(len(interior_regions)):
    region = interior_regions[i]
    interior_polygon = region[0]
    if len(inside_polygon(interior_polygon, bounding_polygon,
                   closed = True, verbose = False)) <> len(interior_polygon):
        print 'WARNING: interior polygon %d is outside bounding polygon' %(i)
        count += 1

if count == 0:
    print 'interior regions OK'
else:
    print 'check out your interior polygons'
    print 'check %s in production directory' %figname
    import sys; sys.exit()
"""
#-------------------------------------

print 'start create mesh from regions'
meshname = project.meshname + '.msh'
create_mesh_from_regions(project.bounding_polygon,
                         boundary_tags={'back': [7, 8], 'side': [0, 6],
                                        'ocean': [1, 2, 3, 4, 5]},
                         maximum_triangle_area=200000,
                         interior_regions=interior_regions,
                         filename=meshname,
                         use_cache=True,
                         verbose=True)


#-------------------------------------------------------------------------
# Setup computational domain
#-------------------------------------------------------------------------
domain = Domain(meshname, use_cache=True, verbose=True)
print domain.statistics()
domain.set_name(project.basename)
domain.set_datadir(project.outputtimedir)
domain.set_default_order(2)
domain.set_minimum_storable_height(0.01) # Don't store anything less than 1cm



#-------------------------------------------------------------------------
# Setup initial conditions
#-------------------------------------------------------------------------
tide = 0.
domain.set_quantity('stage', tide)
domain.set_quantity('friction', 0.0) 
domain.set_quantity('elevation', 
                    filename = project.datadir + project.basename + '.pts',
                    use_cache = False,
                    verbose = True,
                    alpha = 0.1)


#-------------------------------------------------------------------------
# Setup boundary conditions 
#-------------------------------------------------------------------------


print 'Available boundary tags', domain.get_boundary_tags()

from anuga.shallow_water.data_manager import urs2sww

urs2sww(boundary_file, verbose='true')

Bf = File_boundary(source_dir + project.boundary_basename + '.sww',
                   domain, verbose = True)
Br = Reflective_boundary(domain)
Bd = Dirichlet_boundary([tide,0,0])
domain.set_boundary({'back': Br,
                     'side': Bd,
                     'ocean': Bf}) 

#----------------------------------------------------------------------------
# Evolve system through time
#----------------------------------------------------------------------------
import time
t0 = time.time()

for t in domain.evolve(yieldstep = 120, finaltime = 28800):
    domain.write_time()
    domain.write_boundary_statistics(tags = 'ocean')     
    
print 'That took %.2f seconds' %(time.time()-t0)

